/*
 * entities.c : implementation for the XML entities handking
 */

#include <stdio.h>
#include <malloc.h>
#include "entities.h"

/*
 * An unit of storage for an entity, contains the string, the value
 * and the linkind data needed for the linking in the hash table.
 */

typedef struct xmlEntity {
    struct xmlEntity *nextHash;	/* Next entity in the hash table */
    CHAR value;			/* The entity CHAR equivalent */
    const CHAR *id;		/* The entity name */
} xmlEntity, *xmlEntityPtr;

/*
 * ALl entities are stored in an hash table associated to the document
 */

#define XML_NB_ENTITIES_HASH	256

typedef struct xmlEntitiesHashTable {
    int nb_entities;		/* number of elements stored */
    xmlEntityPtr table[XML_NB_ENTITIES_HASH]; /* the hash table itself */
} xmlEntitiesHashTable, *xmlEntitiesHashTablePtr;

/*
 * xmlCreateEntity : create a new entity record.
 */
xmlEntityPtr xmlCreateEntity(CHAR value, const CHAR *id) {
    xmlEntityPtr ret;

    ret = (xmlEntityPtr) malloc(sizeof(xmlEntity));
    if (ret == NULL) {
        fprintf(stderr, "xmlCreateEntity : malloc(%d) failed\n",
	        sizeof(xmlEntity));
    }
    ret->value = value;
    ret->id = xmlStrdup(id);
    ret->nextHash = NULL;
    return(ret);
}

/*
 * xmlFreeEntity : clean-up an entity record.
 */

void xmlFreeEntity(xmlEntityPtr entity) {
    if (entity == NULL) return;

    entity->value = (CHAR) -1;
    if (entity->id != NULL)
	free((char *) entity->id);
}

/*
 * xmlAddEntity : register a new entity for this document (DTD ???).
 */
void xmlAddEntity(xmlDocPtr doc, CHAR value, const CHAR *id) {
}

/*
 * xmlGetEntity : do an entity lookup in the hash table and
 *       returns the corrsponding CHAR, if found, zero otherwise.
 */
CHAR xmlGetEntity(xmlDocPtr doc, const CHAR *id) {
}

/*
 * xmlSubstituteEntities : do a global entities lookup on a input string
 *        and returns a duplicate after the entities substitution.
 */
CHAR *xmlSubstituteEntities(xmlDocPtr doc, const CHAR *input) {
}

/*
 * xmlCreateEntitiesHashTable : create and initialize an enmpty hash table
 */
xmlEntitiesHashTablePtr xmlCreateEntitiesHashTable(void) {
    xmlEntitiesHashTablePtr ret;
    int i;

    ret = (xmlEntitiesHashTablePtr) 
         malloc(sizeof(xmlEntitiesHashTable));
    if (ret == NULL) {
        fprintf(stderr, "xmlCreateEntitiesHashTable : malloc(%d) failed\n",
	        sizeof(xmlEntitiesHashTable));
        return(NULL);
    }
    for (i = 0;i < XML_NB_ENTITIES_HASH;i++)
        ret->table[i] = NULL;
    return(ret);
}

/*
 * xmlFreeEntitiesHashTable : clean up and free an entities hash table.
 */
void xmlFreeEntitiesHashTable(xmlEntitiesHashTablePtr table) {
    xmlEntityPtr cur, next;
    int i;

    if (table == NULL) return;

    for (i = 0;i < XML_NB_ENTITIES_HASH;i++) {
        cur = table->table[i];
        while (cur != NULL) {
	    next = cur->nextHash;
            xmlFreeEntity(cur);
	    cur = next;
	}
	table->table[i] = (void *) 0xdeadbeef;
    }
    free(table);
}

